searchNodes=[{"type":"module","title":"exec","doc":"OS shell command runner. It communicates with a separate C++ port process exec-port spawned by this module, which is responsible for starting, killing, listing, terminating, and notifying of state changes. The port program serves as a middle-man between the OS and the virtual machine to carry out OS-specific low-level process control. The Erlang/C++ protocol is described in the exec.cpp file. The exec application can execute tasks by impersonating as a different effective user. This impersonation can be accomplished in one of the following two ways (assuming that the emulator is not running as root : Having the user account running the erlang emulator added to the /etc/sudoers file, so that it can execute exec-port task as root . (Preferred option) Setting root ownership on exec-port , and setting the SUID bit: chown root:root exec-port; chmod 4755 exec-port . (This option is discouraged as it's less secure). In either of these two cases, exec:start_link/2 must be started with options [root, {user, User}, {limit_users, Users}] , so that exec-port process will not actually run as root but will switch to the effective User , and set the kernel capabilities so that it's able to start processes as other effective users given in the Users list and adjust process priorities. Though, in the initial design, exec prohibited such use, upon user requests a feature was added (in order to support docker deployment and CI testing) to be able to execute exec-port as root without switching the effective user to anying other than root . To accomplish this use the following options to start exec : [root, {user, &quot;root&quot;}, {limit_users, [&quot;root&quot;]}] . At exit the port program makes its best effort to perform clean shutdown of all child OS processes. Every started OS process is linked to a spawned light-weight Erlang process returned by the run/2, run_link/2 command. The application ensures that termination of spawned OsPid leads to termination of the associated Erlang Pid, and vice versa.","ref":"exec.html"},{"type":"function","title":"exec.debug/1","doc":"Set debug level of the port process.","ref":"exec.html#debug/1"},{"type":"function","title":"exec.kill/2","doc":"Send a Signal to a child Pid , OsPid or an Erlang Port .","ref":"exec.html#kill/2"},{"type":"function","title":"exec.manage/2","doc":"","ref":"exec.html#manage/2"},{"type":"function","title":"exec.ospid/1","doc":"Get OsPid of the given Erlang Pid . The Pid must be created previously by running the run/2 or run_link/2 commands.","ref":"exec.html#ospid/1"},{"type":"function","title":"exec.pid/1","doc":"Get Pid of the given OsPid . The OsPid must be created previously by running the run/2 or run_link/2 commands.","ref":"exec.html#pid/1"},{"type":"function","title":"exec.pty_opts/2","doc":"Set the pty terminal options of the OS process identified by OsPid . The process must have been created with the pty option.","ref":"exec.html#pty_opts/2"},{"type":"function","title":"exec.run/2","doc":"","ref":"exec.html#run/2"},{"type":"function","title":"exec.run/3","doc":"Run an external program. OsPid is the OS process identifier of the new process. If sync is specified in Options the return value is {ok, Status} where Status is OS process exit status. The Status can be decoded with status/1 to determine the process's exit code and if it was killed by signal.","ref":"exec.html#run/3"},{"type":"function","title":"exec.run_link/2","doc":"","ref":"exec.html#run_link/2"},{"type":"function","title":"exec.run_link/3","doc":"Equivalent to run / 2. Run an external program and link to the OsPid. If OsPid exits, the calling process will be killed or if it's trapping exits, it'll get {'EXIT', OsPid, Status} message. If the calling process dies the OsPid will be killed. The Status can be decoded with status/1 to determine the process's exit code and if it was killed by signal.","ref":"exec.html#run_link/3"},{"type":"function","title":"exec.send/2","doc":"Send Data to stdin of the OS process identified by OsPid . Sending eof instead of binary Data causes close of stdin of the corresponding process. Data sent to closed stdin is ignored.","ref":"exec.html#send/2"},{"type":"function","title":"exec.setpgid/2","doc":"Change group ID of a given OsPid to Gid .","ref":"exec.html#setpgid/2"},{"type":"function","title":"exec.signal/1","doc":"Convert a signal number to atom","ref":"exec.html#signal/1"},{"type":"function","title":"exec.signal_to_int/1","doc":"","ref":"exec.html#signal_to_int/1"},{"type":"function","title":"exec.start/0","doc":"Equivalent to start_link / 1. Start of an external program manager without supervision. Note that the port program requires SHELL environment variable to be set.","ref":"exec.html#start/0"},{"type":"function","title":"exec.start/1","doc":"","ref":"exec.html#start/1"},{"type":"function","title":"exec.start_link/1","doc":"Supervised start an external program manager. Note that the port program requires SHELL environment variable to be set.","ref":"exec.html#start_link/1"},{"type":"function","title":"exec.status/1","doc":"Decode the program's exit_status. If the program exited by signal the function returns {signal, Signal, Core} where the Signal is the signal number or atom, and Core indicates if the core file was generated.","ref":"exec.html#status/1"},{"type":"function","title":"exec.stop/1","doc":"Terminate a managed Pid , OsPid , or Port process. The OS process is terminated gracefully. If it was given a {kill, Cmd} option at startup, that command is executed and a timer is started. If the program doesn't exit, then the default termination is performed. Default termination implies sending a SIGTERM command followed by SIGKILL in 5 seconds, if the program doesn't get killed.","ref":"exec.html#stop/1"},{"type":"function","title":"exec.stop_and_wait/2","doc":"Terminate a managed Pid , OsPid , or Port process, like stop/1 , and wait for it to exit.","ref":"exec.html#stop_and_wait/2"},{"type":"function","title":"exec.which_children/0","doc":"Get a list of children managed by port program.","ref":"exec.html#which_children/0"},{"type":"function","title":"exec.winsz/3","doc":"Set the pty terminal Rows and Cols of the OS process identified by OsPid . The process must have been created with the pty option.","ref":"exec.html#winsz/3"},{"type":"type","title":"exec.cmd/0","doc":"Command to be executed. If specified as a string, the specified command will be executed through the shell. The current shell is obtained from environment variable SHELL . This can be useful if you are using Erlang primarily for the enhanced control flow it offers over most system shells and still want convenient access to other shell features such as shell pipes, filename wildcards, environment variable expansion, and expansion of ~ to a user's home directory. All command arguments must be properly escaped including whitespace and shell metacharacters. Any part of the command string can contain unicode characters. Warning: Executing shell commands that incorporate unsanitized input from an untrusted source makes a program vulnerable to shell injection , a serious security flaw which can result in arbitrary command execution. For this reason, the use of shell is strongly discouraged in cases where the command string is constructed from external input: 1 &gt; { ok , Filename } = io : read ( &quot;Enter filename: &quot; ) . Enter filename : &quot;non_existent; rm -rf / #&quot; . { ok , &quot;non_existent; rm -rf / #&quot; } 2 &gt; exec ( Filename , [ ] ) % Argh!!! This is not good! When command is given in the form of a list of strings, it is passed to execve(3) library call directly without involving the shell process, so the list of strings represents the program to be executed given with a full path, followed by the list of arguments (e.g. [&quot;/bin/echo&quot;, &quot;ok&quot;] ). In this case all shell-based features are disabled and there's no shell injection vulnerability.","ref":"exec.html#t:cmd/0"},{"type":"type","title":"exec.cmd_option/0","doc":"Command options: monitor Set up a monitor for the spawned process. The monitor is not a standard erlang:montior/2 function call, but it's emulated by ensuring that the monitoring process receives notification in the form: {'DOWN', OsPid::integer(), process, Pid::pid(), Reason} . If the Reason is normal , then process exited with status 0 , otherwise there was an error. If the Reason is {status, Status} the returned Status can be decoded with status/1 to determine the exit code of the process and if it was killed by signal. sync Block the caller until the OS command exits {executable, Executable::string()} Specifies a replacement program to execute. It is very seldom needed. When the port program executes a child process using execve(3) call, the call takes the following arguments: (Executable, Args, Env) . When Cmd argument passed to the run/2 function is specified as the list of strings, the executable replaces the first parameter in the call, and the original args provided in the Cmd parameter are passed as as the second parameter. Most programs treat the program specified by args as the command name, which can then be different from the program actually executed. On Unix, the args name becomes the display name for the executable in utilities such as ps . If Cmd argument passed to the run/2 function is given as a string, on Unix the Executable specifies a replacement shell for the default /bin/sh . {cd, WorkDir} Working directory {env, Env :: [{Name,Value}|string()|clear]} List of &quot;VAR=VALUE&quot; environment variables or list of {Name, Value} tuples or strings (like &quot;NAME=VALUE&quot;) or clear . clear will clear environment of a spawned child OS process (so that it doesn't inherit parent's environment). If Value is false then the Var env variable is unset. {kill, KillCmd} This command will be used for killing the process. After a 5-sec timeout if the process is still alive, it'll be killed with SIGKILL. The kill command will have a CHILD_PID environment variable set to the pid of the process it is expected to kill. If the kill option is not specified, by default first the command is sent a SIGTERM signal, followed by SIGKILL after a default timeout. {kill_timeout, Sec::integer()} Number of seconds to wait after issuing a SIGTERM or executing the custom kill command (if specified) before killing the process with the SIGKILL signal kill_group At process exit kill the whole process group associated with this pid. The process group is obtained by the call to getpgid(3). {group, GID} Sets the effective group ID of the spawned process. The value 0 means to create a new group ID equal to the OS pid of the process. {user, RunAsUser} When exec-port was compiled with capability (Linux) support enabled and has a suid bit set, it's capable of running commands with a different RunAsUser effective user. Passing &quot;root&quot; value of RunAsUser is prohibited. {success_exit_code, IntExitCode} On success use IntExitCode return value instead of default 0. {nice, Priority} Set process priority between -20 and 20. Note that negative values can be specified only when exec-port is started with a root suid bit set. stdin | {stdin, null | close | Filename} Enable communication with an OS process via its stdin . The input to the process is sent by exec:send(OsPid, Data) . When specified as a tuple, null means redirection from /dev/null , close means to close stdin stream, and Filename means to take input from file. stdout Same as {stdout, self()} . stderr Same as {stderr, self()} . {stdout, output_device()} Redirect process's standard output stream {stderr, output_device()} Redirect process's standard error stream {stdout | stderr, Filename::string(), [output_dev_opt()]} Redirect process's stdout/stderr stream to file {winsz, {Rows, Cols}} Set the (psudo) terminal's dimensions of rows and columns pty Use pseudo terminal for the process's stdin, stdout and stderr pty_echo Allow the pty to run in echo mode, disabled by default debug Same as {debug, 1} {debug, Level::integer()} Enable debug printing in port program for this command","ref":"exec.html#t:cmd_option/0"},{"type":"type","title":"exec.cmd_options/0","doc":"","ref":"exec.html#t:cmd_options/0"},{"type":"type","title":"exec.exec_option/0","doc":"Options passed to the exec process at startup. They can be specified in the sys.config file for the erlexec application to customize application startup. debug Same as {debug, 1} {debug, Level} Enable port-programs debug trace at Level . verbose Enable verbose prints of the Erlang process. root | {root, Boolean} Allow running child processes as root. {args, Args} Append Args to the port command. {alarm, Secs} Give Secs deadline for the port program to clean up child pids before exiting {user, User} When the port program was compiled with capability (Linux) support enabled, and is owned by root with a a suid bit set, this option must be specified so that upon startup the port program is running under the effective user different from root. This is a security measure that will also prevent the port program to execute root commands. {limit_users, LimitUsers} Limit execution of external commands to these set of users. This option is only valid when the port program is owned by root. {portexe, Exe} Provide an alternative location of the port program. This option is useful when this application is stored on NFS and the port program needs to be copied locally so that root suid bit can be set. {env, Env} Extend environment of the port program by using Env specification. Env should be a list of tuples {Name, Val} , where Name is the name of an environment variable, and Val is the value it is to have in the spawned port process. If Val is false , then the Name environment variable is unset.","ref":"exec.html#t:exec_option/0"},{"type":"type","title":"exec.exec_options/0","doc":"","ref":"exec.html#t:exec_options/0"},{"type":"type","title":"exec.osgid/0","doc":"Representation of OS group ID.","ref":"exec.html#t:osgid/0"},{"type":"type","title":"exec.ospid/0","doc":"Representation of OS process ID.","ref":"exec.html#t:ospid/0"},{"type":"type","title":"exec.output_dev_opt/0","doc":"Output device option: null Suppress output. close Close file descriptor for writing. print A debugging convenience device that prints the output to the console shell Filename Save output to file by overwriting it. pid() Redirect output to this pid. fun((Stream, OsPid, Data) -&gt; none()) Execute this callback on receiving output data","ref":"exec.html#t:output_dev_opt/0"},{"type":"type","title":"exec.output_file_opt/0","doc":"Defines file opening attributes: append Open the file in append mode {mode, Mode} File creation access mode specified in base 8 (e.g. 8#0644)","ref":"exec.html#t:output_file_opt/0"},{"type":"type","title":"exec.pty_opt/0","doc":"Pty options, see: https://man7.org/linux/man-pages/man3/termios.3.html https://datatracker.ietf.org/doc/html/rfc4254#section-8 {tty_char(), Byte} A special character with value from 0 to 255 {tty_mode(), Enable} Enable/disable a tty mode {tty_speed(), Speed} Specify input or output baud rate. Provided for completeness. Not useful for pseudo terminals.","ref":"exec.html#t:pty_opt/0"},{"type":"type","title":"exec.pty_opts/0","doc":"List of pty options.","ref":"exec.html#t:pty_opts/0"},{"type":"type","title":"exec.tty_char/0","doc":"","ref":"exec.html#t:tty_char/0"},{"type":"type","title":"exec.tty_mode/0","doc":"","ref":"exec.html#t:tty_mode/0"},{"type":"type","title":"exec.tty_speed/0","doc":"","ref":"exec.html#t:tty_speed/0"},{"type":"module","title":"exec_app","doc":"This module implements application and supervisor behaviors of the exec application.","ref":"exec_app.html"},{"type":"extras","title":"Overview","doc":"Author Serge Aleynikov &lt;saleyn(at)gmail.com&gt;","ref":"readme.html"},{"type":"extras","title":"Overview - Summary","doc":"Execute and control OS processes from Erlang/OTP. This project implements an Erlang application with a C++ port program that gives light-weight Erlang processes fine-grain control over execution of OS processes. The following features are supported: Start/stop OS commands and get their OS process IDs, and termination reason (exit code, signal number, core dump status). Support OS commands with unicode characters. Manage/monitor externally started OS processes. Execute OS processes synchronously and asynchronously. Set OS command's working directory, environment, process group, effective user, process priority. Provide custom termination command for killing a process or relying on default SIGTERM/SIGKILL behavior. Specify custom timeout for SIGKILL after the termination command or SIGTERM was executed and the running OS child process is still alive. Link Erlang processes to OS processes (via intermediate Erlang Pids that are linked to an associated OS process), so that the termination of an OS process results in termination of an Erlang Pid, and vice versa. Monitor termination of OS processes using erlang:monitor/2 . Terminate all processes belonging to an OS process group. Kill processes belonging to an OS process group at process exit. Communicate with an OS process via its STDIN. Redirect STDOUT and STDERR of an OS process to a file, erlang process, or a custom function. When redirected to a file, the file can be open in append/truncate mode, and given at creation an access mask. Run interactive processes with psudo-terminal pty support. Support all RFC4254 pty psudo-terminal options defined in section-8 of the spec. Execute OS processes under different user credentials (using Linux capabilities). Perform proper cleanup of OS child processes at port program termination time. This application provides significantly better control over OS processes than built-in erlang:open_port/2 command with a {spawn, Command} option, and performs proper OS child process cleanup when the emulator exits. The erlexec application has been in production use by Erlang and Elixir systems, and is considered stable.","ref":"readme.html#summary"},{"type":"extras","title":"Overview - Donations","doc":"If you find this project useful, please donate to: Bitcoin: 12pt8TcoMWMkF6iY66VJQk95ntdN4pFihg Ethereum: 0x268295486F258037CF53E504fcC1E67eba014218","ref":"readme.html#donations"},{"type":"extras","title":"Overview - Supported Platforms","doc":"Linux, Solaris, FreeBSD, OpenBSD, MacOS X","ref":"readme.html#supported-platforms"},{"type":"extras","title":"Overview - DOCUMENTATION","doc":"See https://hexdocs.pm/erlexec/readme.html","ref":"readme.html#documentation"},{"type":"extras","title":"Overview - USAGE","doc":"Erlang: import as a dependency Add dependency in rebar.config : { deps , [ % ... { erlexec , &quot;~&gt; 2.0&quot; } ] } . Include in your *.app.src : { applications , [ kernel , stdlib , % ... erlexec ] } Elixir: import as a dependency defp deps do [ # ... { :erlexec , &quot;~&gt; 2.0&quot; } ] end","ref":"readme.html#usage"},{"type":"extras","title":"Overview - BUILDING FROM SOURCE","doc":"Make sure you have rebar or rebar3 installed locally and the rebar script is in the path. If you are deploying the application on Linux and would like to take advantage of exec-port running tasks using effective user IDs different from the real user ID that started exec-port, then either make sure that libcap-dev[el] library is installed or make sure that the user running the port program has sudo rights. OS-specific libcap-dev installation instructions: Fedora, CentOS: &quot;yum install libcap-devel&quot; Ubuntu: &quot;apt-get install libcap-dev&quot; $ git clone git@github.com:saleyn/erlexec.git $ make # NOTE: for disabling optimized build of exec-port, do the following instead: $ OPTIMIZE=0 make By default port program's implementation uses poll(2) call for event demultiplexing. If you prefer to use select(2) , set the following environment variable: $ USE_POLL=0 make","ref":"readme.html#building-from-source"},{"type":"extras","title":"Overview - LICENSE","doc":"The program is distributed under BSD license. Copyright (c) 2003 Serge Aleynikov","ref":"readme.html#license"},{"type":"extras","title":"Overview - Architecture","doc":"* -- -- -- -- -- -- -- -- -- -- -- -- -- - * | + -- -- + + -- -- + + -- -- + | | | Pid1 | | Pid2 | | PidN | | Erlang light - weight Pids associated | + -- -- + + -- -- + + -- -- + | one - to - one with managed OsPids | \\ | / | | \\ | / | | \\ | / ( links ) | | + -- -- -- + | | | exec | | Exec application running in Erlang VM | + -- -- -- + | | Erlang VM | | * -- -- -- -- -- -- - + -- -- -- -- -- -- - * | + -- -- -- -- -- - + | exec - port | Port program ( separate OS process ) + -- -- -- -- -- - + / | \\ ( optional stdin / stdout / stderr pipes ) / | \\ + -- -- -- + + -- -- -- + + -- -- -- + | OsPid1 | | OsPid2 | | OsPidN | Managed Child OS processes + -- -- -- + + -- -- -- + + -- -- -- +","ref":"readme.html#architecture"},{"type":"extras","title":"Overview - Configuration Options","doc":"See description of types in {@link exec:exec_options()}. The exec-port program requires the SHELL variable to be set. If you are running Erlang inside a docker container, you might need to ensure that SHELL is properly set prior to starting the emulator.","ref":"readme.html#configuration-options"},{"type":"extras","title":"Overview - Examples","doc":"Starting/stopping an OS process 1 &gt; exec : start ( ) . % Start the port program. { ok , &lt; 0.32 . 0 &gt; } 2 &gt; { ok , _ , I } = exec : run_link ( &quot;sleep 1000&quot; , [ ] ) . % Run a shell command to sleep for 1000s. { ok , &lt; 0.34 . 0 &gt; , 23584 } 3 &gt; exec : stop ( I ) . % Kill the shell command. ok % Note that this could also be accomplished % by doing exec:stop(pid(0,34,0)). In Elixir: iex(1)&gt; :exec . start { :ok , # PID &lt; 0 . 112 . 0 &gt; } iex(2)&gt; :exec . run ( &quot;echo ok&quot; , [ :sync , :stdout ] ) { :ok , [ stdout : [ &quot;ok \\n &quot; ] ] } Clearing environment or unsetting an env variable of the child process %% Clear environment with {env, [clear]} option: 10 &gt; f ( Bin ) , { ok , [ { stdout , [ Bin ] } ] } = exec : run ( &quot;env&quot; , [ sync , stdout , { env , [ clear ] } ] ) , p ( re : split ( Bin , &lt;&lt; &quot;\\n&quot; &gt;&gt; ) ) . [ &lt;&lt; &quot;PWD=/home/...&quot; &gt;&gt; , &lt;&lt; &quot;SHLVL=0&quot; &gt;&gt; , &lt;&lt; &quot;_=/usr/bin/env&quot; &gt;&gt; , &lt;&lt; &gt;&gt; ] ok %% Clear env and add a &quot;TEST&quot; env variable: 11 &gt; f ( Bin ) , { ok , [ { stdout , [ Bin ] } ] } = exec : run ( &quot;env&quot; , [ sync , stdout , { env , [ clear , { &quot;TEST&quot; , &quot;xxx&quot; } ] } ] ) , p ( re : split ( Bin , &lt;&lt; &quot;\\n&quot; &gt;&gt; ) ) . [ &lt;&lt; &quot;PWD=/home/...&quot; &gt;&gt; , &lt;&lt; &quot;SHLVL=0&quot; &gt;&gt; , &lt;&lt; &quot;_=/usr/bin/env&quot; &gt;&gt; , &lt;&lt; &quot;TEST=xxx&quot; &gt;&gt; , &lt;&lt; &gt;&gt; ] %% Unset an &quot;EMU&quot; env variable: 11 &gt; f ( Bin ) , { ok , [ { stdout , [ Bin ] } ] } = exec : run ( &quot;env&quot; , [ sync , stdout , { env , [ { &quot;EMU&quot; , false } ] } ] ) , p ( re : split ( Bin , &lt;&lt; &quot;\\n&quot; &gt;&gt; ) ) . [ . . . ] ok Running exec-port as another effective user In order to be able to use this feature the current user must either have sudo rights or the exec-port file must be owned by root and have the SUID bit set (use: chown root:root exec-port; chmod 4555 exec-port ): $ ll priv/x86_64-unknown-linux-gnu/exec-port -rwsr-xr-x 1 root root 777336 Dec 8 10:02 ./priv/x86_64-unknown-linux-gnu/exec-port If the effective user doesn't have rights to access the exec-port program in the real user's directory, then the exec-port can be copied to some shared location, which will be specified at startup using {portexe, &quot;/path/to/exec-port&quot;} . $ cp $( find . - name exec - port ) / tmp $ chmod 755 / tmp / exec - port $ whoami serge $ erl 1 &gt; exec : start ( [ { user , &quot;wheel&quot; } , { portexe , &quot;/tmp/exec-port&quot; } ] ) . % Start the port program as effective user &quot;wheel&quot;. { ok , &lt; 0.32 . 0 &gt; } $ ps haxo user , comm | grep exec - port wheel exec - port Allowing exec-port to run commands as other effective users In order to be able to use this feature the current user must either have sudo rights or the exec-port file must have the SUID bit set, and the exec-port file must have the capabilities set as described in the &quot;Build&quot; section above. The port program will initially be started as root , and then it will switch the effective user to {user, User} and set process capabilities to cap_setuid,cap_kill,cap_sys_nice . After that it'll allow to run child programs under effective users listed in the {limit_users, Users} option. $ whoami serge $ erl 1 &gt; Opts = [ root , { user , &quot;wheel&quot; } , { limit_users , [ &quot;alex&quot; , &quot;guest&quot; ] } ] , 2 &gt; exec : start ( Opts ) . % Start the port program as effective user &quot;wheel&quot; % and allow it to execute commands as &quot;alex&quot; or &quot;guest&quot;. { ok , &lt; 0.32 . 0 &gt; } 3 &gt; exec : run ( &quot;whoami&quot; , [ sync , stdout , { user , &quot;alex&quot; } ] ) . % Command is executed under effective user &quot;alex&quot; { ok , [ { stdout , [ &lt;&lt; &quot;alex\\n&quot; &gt;&gt; ] } ] } $ ps haxo user , comm | grep exec - port wheel exec - port Running the port program as root While running the port program as root is highly discouraged, since it opens a security hole that gives users an ability to damage the system, for those who might need such an option, here is how to get it done (PROCEED AT YOUR OWN RISK!!!). Note: in this case exec would use sudo exec-port to run it as root or the exec-port must have the SUID bit set (4555) and be owned by root . The other (DANGEROUS and firmly DISCOURAGED!!!) alternative is to run erl as root : $ whoami serge # Make sure the exec - port can run as root : $ sudo _ build / default / lib / erlexec / priv / * / exec - port -- whoami root $ erl 1 &gt; exec : start ( [ root , { user , &quot;root&quot; } , { limit_users , [ &quot;root&quot; ] } ] ) . 2 &gt; exec : run ( &quot;whoami&quot; , [ sync , stdout ] ) . { ok , [ { stdout , [ &lt;&lt; &quot;root\\n&quot; &gt;&gt; ] } ] } $ ps haxo user , comm | grep exec - port root exec - port Killing an OS process Note that killing a process can be accomplished by running kill(3) command in an external shell, or by executing exec:kill/2. 1 &gt; f ( I ) , { ok , _ , I } = exec : run_link ( &quot;sleep 1000&quot; , [ ] ) . { ok , &lt; 0.37 . 0 &gt; , 2350 } 2 &gt; exec : kill ( I , 15 ) . ok * * exception error : { exit_status , 15 } % Our shell died because we linked to the % killed shell process via exec:run_link/2. 3 &gt; exec : status ( 15 ) . % Examine the exit status. { signal , 15 , false } % The program got SIGTERM signal and produced % no core file. Using a custom success return code 1 &gt; exec : start_link ( [ ] ) . { ok , &lt; 0.35 . 0 &gt; } 2 &gt; exec : run_link ( &quot;sleep 1&quot; , [ { success_exit_code , 0 } , sync ] ) . { ok , [ ] } 3 &gt; exec : run ( &quot;sleep 1&quot; , [ { success_exit_code , 1 } , sync ] ) . { error , [ { exit_status , 1 } ] } % Note that the command returns exit code 1 Redirecting OS process stdout to a file 7 &gt; f ( I ) , { ok , _ , I } = exec : run_link ( &quot;for i in 1 2 3; do echo \\&quot;Test$i\\&quot;; done&quot; , [ { stdout , &quot;/tmp/output&quot; } ] ) . 8 &gt; io : format ( &quot; ~s &quot; , [ binary_to_list ( element ( 2 , file : read_file ( &quot;/tmp/output&quot; ) ) ) ] ) , file : delete ( &quot;/tmp/output&quot; ) . Test1 Test2 Test3 ok Redirecting OS process stdout to screen, an Erlang process or a custom function 9 &gt; exec : run ( &quot;echo Test&quot; , [ { stdout , print } ] ) . { ok , &lt; 0.119 . 0 &gt; , 29651 } Got stdout from 29651 : &lt;&lt; &quot;Test\\n&quot; &gt;&gt; 10 &gt; exec : run ( &quot;for i in 1 2 3; do sleep 1; echo \\&quot;Iter$i\\&quot;; done&quot; , [ { stdout , fun ( S , OsPid , D ) -&gt; io : format ( &quot;Got ~w from ~w : ~p \\n&quot; , [ S , OsPid , D ] ) end } ] ) . { ok , &lt; 0.121 . 0 &gt; , 29652 } Got stdout from 29652 : &lt;&lt; &quot;Iter1\\n&quot; &gt;&gt; Got stdout from 29652 : &lt;&lt; &quot;Iter2\\n&quot; &gt;&gt; Got stdout from 29652 : &lt;&lt; &quot;Iter3\\n&quot; &gt;&gt; % Note that stdout/stderr options are equivanet to {stdout, self()}, {stderr, self()} 11 &gt; exec : run ( &quot;echo Hello World!; echo ERR!! 1&gt;&amp;2&quot; , [ stdout , stderr ] ) . { ok , &lt; 0.244 . 0 &gt; , 18382 } 12 &gt; flush ( ) . Shell got { stdout , 18382 , &lt;&lt; &quot;Hello World!\\n&quot; &gt;&gt; } Shell got { stderr , 18382 , &lt;&lt; &quot;ERR!!\\n&quot; &gt;&gt; } ok Appending OS process stdout to a file 13 &gt; exec : run ( &quot;for i in 1 2 3; do echo TEST$i; done&quot; , [ { stdout , &quot;/tmp/out&quot; , [ append , { mode , 8#600 } ] } , sync ] ) , file : read_file ( &quot;/tmp/out&quot; ) . { ok , &lt;&lt; &quot;TEST1\\nTEST2\\nTEST3\\n&quot; &gt;&gt; } 14 &gt; exec : run ( &quot;echo Test4; done&quot; , [ { stdout , &quot;/tmp/out&quot; , [ append , { mode , 8#600 } ] } , sync ] ) , file : read_file ( &quot;/tmp/out&quot; ) . { ok , &lt;&lt; &quot;TEST1\\nTEST2\\nTEST3\\nTest4\\n&quot; &gt;&gt; } 15 &gt; file : delete ( &quot;/tmp/out&quot; ) . Setting up a monitor for the OS process &gt; f ( I ) , f ( P ) , { ok , P , I } = exec : run ( &quot;echo ok&quot; , [ { stdout , self ( ) } , monitor ] ) . { ok , &lt; 0.263 . 0 &gt; , 18950 } 16 &gt; flush ( ) . Shell got { stdout , 18950 , &lt;&lt; &quot;ok\\n&quot; &gt;&gt; } Shell got { &#39;DOWN&#39; , 18950 , process , &lt; 0.263 . 0 &gt; , normal } ok Managing an externally started OS process This command allows to instruct erlexec to begin monitoring given OS process and notify Erlang when the process exits. It is also able to send signals to the process and kill it. % Start an externally managed OS process and retrieve its OS PID: 17 &gt; spawn ( fun ( ) -&gt; os : cmd ( &quot;echo $$ &gt; /tmp/pid; sleep 15&quot; ) end ) . &lt; 0.330 . 0 &gt; 18 &gt; f ( P ) , P = list_to_integer ( lists : reverse ( tl ( lists : reverse ( binary_to_list ( element ( 2 , file : read_file ( &quot;/tmp/pid&quot; ) ) ) ) ) ) ) . 19355 % Manage the process and get notified by a monitor when it exits: 19 &gt; exec : manage ( P , [ monitor ] ) . { ok , &lt; 0.334 . 0 &gt; , 19355 } % Wait for monitor notification 20 &gt; f ( M ) , receive M -&gt; M end . { &#39;DOWN&#39; , 19355 , process , &lt; 0.334 . 0 &gt; , { exit_status , 10 } } ok 21 &gt; file : delete ( &quot;/tmp/pid&quot; ) . ok Specifying a custom process shutdown delay in seconds % Execute an OS process (script) that blocks SIGTERM with custom kill timeout, and monitor 22 &gt; f ( I ) , { ok , _ , I } = exec : run ( &quot;trap &#39;&#39; SIGTERM; sleep 30&quot; , [ { kill_timeout , 3 } , monitor ] ) . { ok , &lt; 0.399 . 0 &gt; , 26347 } % Attempt to stop the OS process 23 &gt; exec : stop ( I ) . ok % Wait for its completion 24 &gt; f ( M ) , receive M -&gt; M after 10000 -&gt; timeout end . { &#39;DOWN&#39; , 26347 , process , &lt; 0.403 . 0 &gt; , normal } Specifying a custom kill command for a process % Execute an OS process (script) that blocks SIGTERM, and uses a custom kill command, % which kills it with a SIGINT. Add a monitor so that we can wait for process exit % notification. Note the use of the special environment variable &quot;CHILD_PID&quot; by the % kill command. This environment variable is set by the port program before invoking % the kill command: 2 &gt; f ( I ) , { ok , _ , I } = exec : run ( &quot;trap &#39;&#39; SIGTERM; sleep 30&quot; , [ { kill , &quot;kill -n 2 ${CHILD_PID}&quot; } , { kill_timeout , 2 } , monitor ] ) . { ok , &lt; 0.399 . 0 &gt; , 26347 } % Try to kill by SIGTERM. This does nothing, since the process is blocking SIGTERM: 3 &gt; exec : kill ( I , sigterm ) , f ( M ) , receive M -&gt; M after 0 -&gt; timeout end . timeout % Attempt to stop the OS process 4 &gt; exec : stop ( I ) . ok % Wait for its completion 5 &gt; f ( M ) , receive M -&gt; M after 1000 -&gt; timeout end . { &#39;DOWN&#39; , 26347 , process , &lt; 0.403 . 0 &gt; , normal } Communicating with an OS process via STDIN % Execute an OS process (script) that reads STDIN and echoes it back to Erlang 25 &gt; f ( I ) , { ok , _ , I } = exec : run ( &quot;read x; echo \\&quot;Got: $x\\&quot;&quot; , [ stdin , stdout , monitor ] ) . { ok , &lt; 0.427 . 0 &gt; , 26431 } % Send the OS process some data via its stdin 26 &gt; exec : send ( I , &lt;&lt; &quot;Test data\\n&quot; &gt;&gt; ) . ok % Get the response written to processes stdout 27 &gt; f ( M ) , receive M -&gt; M after 10000 -&gt; timeout end . { stdout , 26431 , &lt;&lt; &quot;Got: Test data\\n&quot; &gt;&gt; } % Confirm that the process exited 28 &gt; f ( M ) , receive M -&gt; M after 10000 -&gt; timeout end . { &#39;DOWN&#39; , 26431 , process , &lt; 0.427 . 0 &gt; , normal } Communicating with an OS process via STDIN and sending end-of-file 2 &gt; Watcher = spawn ( fun F ( ) -&gt; receive Msg -&gt; io : format ( &quot;Got: ~p \\n&quot; , [ Msg ] ) , F ( ) after 60000 -&gt; ok end end ) . &lt; 0.112 . 0 &gt; 3 &gt; f ( Pid ) , f ( OsPid ) , { ok , Pid , OsPid } = exec : run ( &quot;tac&quot; , [ stdin , { stdout , Watcher } , { stderr , Watcher } ] ) . { ok , &lt; 0.114 . 0 &gt; , 26143 } 4 &gt; exec : send ( Pid , &lt;&lt; &quot;foo\\n&quot; &gt;&gt; ) . ok 5 &gt; exec : send ( Pid , &lt;&lt; &quot;bar\\n&quot; &gt;&gt; ) . ok 6 &gt; exec : send ( Pid , &lt;&lt; &quot;baz\\n&quot; &gt;&gt; ) . ok 7 &gt; exec : send ( Pid , eof ) . ok Got : { stdout , 26143 , &lt;&lt; &quot;baz\\nbar\\nfoo\\n&quot; &gt;&gt; } Running OS commands synchronously % Execute an shell script that blocks for 1 second and return its termination code 29 &gt; exec : run ( &quot;sleep 1; echo Test&quot; , [ sync ] ) . % By default all I/O is redirected to /dev/null, so no output is captured { ok , [ ] } % &#39;stdout&#39; option instructs the port program to capture stdout and return it to caller 30 &gt; exec : run ( &quot;sleep 1; echo Test&quot; , [ stdout , sync ] ) . { ok , [ { stdout , [ &lt;&lt; &quot;Test\\n&quot; &gt;&gt; ] } ] } % Execute a non-existing command 31 &gt; exec : run ( &quot;echo1 Test&quot; , [ sync , stdout , stderr ] ) . { error , [ { exit_status , 32512 } , { stderr , [ &lt;&lt; &quot;/bin/bash: echo1: command not found\\n&quot; &gt;&gt; ] } ] } % Capture stdout/stderr of the executed command 32 &gt; exec : run ( &quot;echo Test; echo Err 1&gt;&amp;2&quot; , [ sync , stdout , stderr ] ) . { ok , [ { stdout , [ &lt;&lt; &quot;Test\\n&quot; &gt;&gt; ] } , { stderr , [ &lt;&lt; &quot;Err\\n&quot; &gt;&gt; ] } ] } % Redirect stderr to stdout 33 &gt; exec : run ( &quot;echo Test 1&gt;&amp;2&quot; , [ { stderr , stdout } , stdout , sync ] ) . { ok , [ { stdout , [ &lt;&lt; &quot;Test\\n&quot; &gt;&gt; ] } ] } Running OS commands with/without shell % Execute a command by an OS shell interpreter 34 &gt; exec : run ( &quot;echo ok&quot; , [ sync , stdout ] ) . { ok , [ { stdout , [ &lt;&lt; &quot;ok\\n&quot; &gt;&gt; ] } ] } % Execute an executable without a shell (note that in this case % the full path to the executable is required): 35 &gt; exec : run ( [ &quot;/bin/echo&quot; , &quot;ok&quot; ] , [ sync , stdout ] ) ) . { ok , [ { stdout , [ &lt;&lt; &quot;ok\\n&quot; &gt;&gt; ] } ] } % Execute a shell with custom options 36 &gt; exec : run ( [ &quot;/bin/bash&quot; , &quot;-c&quot; , &quot;echo ok&quot; ] , [ sync , stdout ] ) ) . { ok , [ { stdout , [ &lt;&lt; &quot;ok\\n&quot; &gt;&gt; ] } ] } Running OS commands with pseudo terminal (pty) % Execute a command without a pty 37 &gt; exec : run ( &quot;echo hello&quot; , [ sync , stdout ] ) . { ok , [ { stdout , [ &lt;&lt; &quot;hello\\n&quot; &gt;&gt; ] } ] } % Execute a command with a pty 38 &gt; exec : run ( &quot;echo hello&quot; , [ sync , stdout , pty ] ) . { ok , [ { stdout , [ &lt;&lt; &quot;hello&quot; &gt;&gt; , &lt;&lt; &quot;\\r\\n&quot; &gt;&gt; ] } ] } % Execute a command with pty echo 39 &gt; { ok , P0 , I0 } = exec : run ( &quot;cat&quot; , [ stdin , stdout , { stderr , stdout } , pty , pty_echo ] ) . { ok , &lt; 0.162 . 0 &gt; , 17086 } 40 &gt; exec : send ( I0 , &lt;&lt; &quot;hello&quot; &gt;&gt; ) . ok 41 &gt; flush ( ) . Shell got { stdout , 17086 , &lt;&lt; &quot;hello&quot; &gt;&gt; } ok 42 &gt; exec : send ( I0 , &lt;&lt; &quot;\\n&quot; &gt;&gt; ) . ok 43 &gt; flush ( ) . Shell got { stdout , 17086 , &lt;&lt; &quot;\\r\\n&quot; &gt;&gt; } Shell got { stdout , 17086 , &lt;&lt; &quot;hello\\r\\n&quot; &gt;&gt; } ok 44 &gt; exec : send ( I , &lt;&lt; 3 &gt;&gt; ) . ok 45 &gt; flush ( ) . Shell got { stdout , 17086 , &lt;&lt; &quot;^C&quot; &gt;&gt; } Shell got { &#39;DOWN&#39; , 17086 , process , &lt; 0.162 . 0 &gt; , { exit_status , 2 } } ok % Execute a command with custom pty options 46 &gt; { ok , P1 , I1 } = exec : run ( &quot;cat&quot; , [ stdin , stdout , { stderr , stdout } , { pty , [ { vintr , 2 } ] } , monitor ] ) . { ok , &lt; 0.199 . 0 &gt; , 16662 } 47 &gt; exec : send ( I1 , &lt;&lt; 3 &gt;&gt; ) . ok 48 &gt; flush ( ) . ok 49 &gt; exec : send ( I1 , &lt;&lt; 2 &gt;&gt; ) . ok 50 &gt; flush ( ) . Shell got { &#39;DOWN&#39; , 16662 , process , &lt; 0.199 . 0 &gt; , { exit_status , 2 } } ok Kill a process group at process exit % In the following scenario the process P0 will create a new process group % equal to the OS pid of that process (value = GID). The next two commands % are assigned to the same process group GID. As soon as the P0 process exits % P1 and P2 will also get terminated by signal 15 (SIGTERM): 51 &gt; { ok , P2 , GID } = exec : run ( &quot;sleep 10&quot; , [ { group , 0 } , kill_group ] ) . { ok , &lt; 0.37 . 0 &gt; , 25306 } 52 &gt; { ok , P3 , _ } = exec : run ( &quot;sleep 15&quot; , [ { group , GID } , monitor ] ) . { ok , &lt; 0.39 . 0 &gt; , 25307 } 53 &gt; { ok , P4 , _ } = exec : run ( &quot;sleep 15&quot; , [ { group , GID } , monitor ] ) . { ok , &lt; 0.41 . 0 &gt; , 25308 } 54 &gt; flush ( ) . Shell got { &#39;DOWN&#39; , 25307 , process , &lt; 0.39 . 0 &gt; , { exit_status , 15 } } Shell got { &#39;DOWN&#39; , 25308 , process , &lt; 0.41 . 0 &gt; , { exit_status , 15 } } ok","ref":"readme.html#examples"},{"type":"extras","title":"License","doc":"BSD LICENSE =========== Copyright (C) 2003 Serge Aleynikov &lt;saleyn@gmail.com&gt; All rights reserved. Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met: 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. 3. The names of the authors may not be used to endorse or promote products derived from this software without specific prior written permission. THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JCRAFT, INC. OR ANY CONTRIBUTORS TO THIS SOFTWARE BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.","ref":"license.html"}]